@@ -24,10 +24,8 @@ class Codec(ABC):
 
     @staticmethod
     @abstractmethod
-    def compress(data: bytes) -> tuple[bytes, int]:
-        ...
+    def compress(data: bytes) -> tuple[bytes, int]: ...
 
     @staticmethod
     @abstractmethod
-    def decompress(data: bytes) -> bytes:
-        ...
+    def decompress(data: bytes) -> bytes: ...
