@@ -37,6 +37,7 @@ from cachetools import LRUCache, cached
 from cachetools.keys import hashkey
 from pydantic_core import to_json
 
+from pyiceberg.avro.codecs import AVRO_CODEC_KEY, AvroCompressionCodec
 from pyiceberg.avro.file import AvroFile, AvroOutputFile
 from pyiceberg.conversions import to_bytes
 from pyiceberg.exceptions import ValidationError
@@ -97,6 +98,7 @@ class FileFormat(str, Enum):
     AVRO = "AVRO"
     PARQUET = "PARQUET"
     ORC = "ORC"
+    PUFFIN = "PUFFIN"
 
     @classmethod
     def _missing_(cls, value: object) -> Union[None, str]:
@@ -288,6 +290,129 @@ DATA_FILE_TYPE: Dict[int, StructType] = {
             doc="ID representing sort order for this file",
         ),
     ),
+    3: StructType(
+        NestedField(
+            field_id=134,
+            name="content",
+            field_type=IntegerType(),
+            required=True,
+            doc="File format name: avro, orc, or parquet",
+            initial_default=DataFileContent.DATA,
+        ),
+        NestedField(field_id=100, name="file_path", field_type=StringType(), required=True, doc="Location URI with FS scheme"),
+        NestedField(
+            field_id=101,
+            name="file_format",
+            field_type=StringType(),
+            required=True,
+            doc="File format name: avro, orc, or parquet",
+        ),
+        NestedField(
+            field_id=102,
+            name="partition",
+            field_type=StructType(),
+            required=True,
+            doc="Partition data tuple, schema based on the partition spec",
+        ),
+        NestedField(field_id=103, name="record_count", field_type=LongType(), required=True, doc="Number of records in the file"),
+        NestedField(
+            field_id=104, name="file_size_in_bytes", field_type=LongType(), required=True, doc="Total file size in bytes"
+        ),
+        NestedField(
+            field_id=108,
+            name="column_sizes",
+            field_type=MapType(key_id=117, key_type=IntegerType(), value_id=118, value_type=LongType()),
+            required=False,
+            doc="Map of column id to total size on disk",
+        ),
+        NestedField(
+            field_id=109,
+            name="value_counts",
+            field_type=MapType(key_id=119, key_type=IntegerType(), value_id=120, value_type=LongType()),
+            required=False,
+            doc="Map of column id to total count, including null and NaN",
+        ),
+        NestedField(
+            field_id=110,
+            name="null_value_counts",
+            field_type=MapType(key_id=121, key_type=IntegerType(), value_id=122, value_type=LongType()),
+            required=False,
+            doc="Map of column id to null value count",
+        ),
+        NestedField(
+            field_id=137,
+            name="nan_value_counts",
+            field_type=MapType(key_id=138, key_type=IntegerType(), value_id=139, value_type=LongType()),
+            required=False,
+            doc="Map of column id to number of NaN values in the column",
+        ),
+        NestedField(
+            field_id=125,
+            name="lower_bounds",
+            field_type=MapType(key_id=126, key_type=IntegerType(), value_id=127, value_type=BinaryType()),
+            required=False,
+            doc="Map of column id to lower bound",
+        ),
+        NestedField(
+            field_id=128,
+            name="upper_bounds",
+            field_type=MapType(key_id=129, key_type=IntegerType(), value_id=130, value_type=BinaryType()),
+            required=False,
+            doc="Map of column id to upper bound",
+        ),
+        NestedField(
+            field_id=131, name="key_metadata", field_type=BinaryType(), required=False, doc="Encryption key metadata blob"
+        ),
+        NestedField(
+            field_id=132,
+            name="split_offsets",
+            field_type=ListType(element_id=133, element_type=LongType(), element_required=True),
+            required=False,
+            doc="Splittable offsets",
+        ),
+        NestedField(
+            field_id=135,
+            name="equality_ids",
+            field_type=ListType(element_id=136, element_type=LongType(), element_required=True),
+            required=False,
+            doc="Field ids used to determine row equality in equality delete files.",
+        ),
+        NestedField(
+            field_id=140,
+            name="sort_order_id",
+            field_type=IntegerType(),
+            required=False,
+            doc="ID representing sort order for this file",
+        ),
+        NestedField(
+            field_id=142,
+            name="first_row_id",
+            field_type=LongType(),
+            required=False,
+            doc="The _row_id for the first row in the data file.",
+        ),
+        NestedField(
+            field_id=143,
+            name="referenced_data_file",
+            field_type=StringType(),
+            required=False,
+            doc="Fully qualified location (URI with FS scheme) of a data file that all deletes reference",
+        ),
+        NestedField(
+            field_id=144,
+            name="content_offset",
+            field_type=LongType(),
+            required=False,
+            doc="The offset in the file where the content starts.",
+        ),
+        NestedField(
+            field_id=145,
+            name="content_size_in_bytes",
+            field_type=LongType(),
+            required=False,
+            doc="The length of a referenced content stored in the file; required if content_offset is present",
+        ),
+    ),
 }
 
 
@@ -321,42 +446,85 @@ def data_file_with_partition(partition_type: StructType, format_version: TableVe
 
 
 class DataFile(Record):
-    __slots__ = (
-        "content",
-        "file_path",
-        "file_format",
-        "partition",
-        "record_count",
-        "file_size_in_bytes",
-        "column_sizes",
-        "value_counts",
-        "null_value_counts",
-        "nan_value_counts",
-        "lower_bounds",
-        "upper_bounds",
-        "key_metadata",
-        "split_offsets",
-        "equality_ids",
-        "sort_order_id",
-        "spec_id",
-    )
-    content: DataFileContent
-    file_path: str
-    file_format: FileFormat
-    partition: Record
-    record_count: int
-    file_size_in_bytes: int
-    column_sizes: Dict[int, int]
-    value_counts: Dict[int, int]
-    null_value_counts: Dict[int, int]
-    nan_value_counts: Dict[int, int]
-    lower_bounds: Dict[int, bytes]
-    upper_bounds: Dict[int, bytes]
-    key_metadata: Optional[bytes]
-    split_offsets: Optional[List[int]]
-    equality_ids: Optional[List[int]]
-    sort_order_id: Optional[int]
-    spec_id: int
+    @classmethod
+    def from_args(cls, _table_format_version: TableVersion = DEFAULT_READ_VERSION, **arguments: Any) -> DataFile:
+        struct = DATA_FILE_TYPE[_table_format_version]
+        return super()._bind(struct, **arguments)
+
+    @property
+    def content(self) -> DataFileContent:
+        return self._data[0]
+
+    @property
+    def file_path(self) -> str:
+        return self._data[1]
+
+    @property
+    def file_format(self) -> FileFormat:
+        return self._data[2]
+
+    @property
+    def partition(self) -> Record:
+        return self._data[3]
+
+    @property
+    def record_count(self) -> int:
+        return self._data[4]
+
+    @property
+    def file_size_in_bytes(self) -> int:
+        return self._data[5]
+
+    @property
+    def column_sizes(self) -> Dict[int, int]:
+        return self._data[6]
+
+    @property
+    def value_counts(self) -> Dict[int, int]:
+        return self._data[7]
+
+    @property
+    def null_value_counts(self) -> Dict[int, int]:
+        return self._data[8]
+
+    @property
+    def nan_value_counts(self) -> Dict[int, int]:
+        return self._data[9]
+
+    @property
+    def lower_bounds(self) -> Dict[int, bytes]:
+        return self._data[10]
+
+    @property
+    def upper_bounds(self) -> Dict[int, bytes]:
+        return self._data[11]
+
+    @property
+    def key_metadata(self) -> Optional[bytes]:
+        return self._data[12]
+
+    @property
+    def split_offsets(self) -> Optional[List[int]]:
+        return self._data[13]
+
+    @property
+    def equality_ids(self) -> Optional[List[int]]:
+        return self._data[14]
+
+    @property
+    def sort_order_id(self) -> Optional[int]:
+        return self._data[15]
+
+    # Spec ID should not be stored in the file
+    _spec_id: int
+
+    @property
+    def spec_id(self) -> int:
+        return self._spec_id
+
+    @spec_id.setter
+    def spec_id(self, value: int) -> None:
+        self._spec_id = value
 
     def __setattr__(self, name: str, value: Any) -> None:
         """Assign a key/value to a DataFile."""
@@ -365,12 +533,6 @@ class DataFile(Record):
             value = FileFormat[value]
         super().__setattr__(name, value)
 
-    def __init__(self, format_version: TableVersion = DEFAULT_READ_VERSION, *data: Any, **named_data: Any) -> None:
-        super().__init__(
-            *data,
-            **{"struct": DATA_FILE_TYPE[format_version], **named_data},
-        )
-
     def __hash__(self) -> int:
         """Return the hash of the file path."""
         return hash(self.file_path)
@@ -396,6 +558,13 @@ MANIFEST_ENTRY_SCHEMAS = {
         NestedField(4, "file_sequence_number", LongType(), required=False),
         NestedField(2, "data_file", DATA_FILE_TYPE[2], required=True),
     ),
+    3: Schema(
+        NestedField(0, "status", IntegerType(), required=True),
+        NestedField(1, "snapshot_id", LongType(), required=False),
+        NestedField(3, "sequence_number", LongType(), required=False),
+        NestedField(4, "file_sequence_number", LongType(), required=False),
+        NestedField(2, "data_file", DATA_FILE_TYPE[3], required=True),
+    ),
 }
 
 MANIFEST_ENTRY_SCHEMAS_STRUCT = {format_version: schema.as_struct() for format_version, schema in MANIFEST_ENTRY_SCHEMAS.items()}
@@ -411,53 +580,49 @@ def manifest_entry_schema_with_data_file(format_version: TableVersion, data_file
 
 
 class ManifestEntry(Record):
-    __slots__ = ("status", "snapshot_id", "sequence_number", "file_sequence_number", "data_file")
-    status: ManifestEntryStatus
-    snapshot_id: Optional[int]
-    sequence_number: Optional[int]
-    file_sequence_number: Optional[int]
-    data_file: DataFile
+    @classmethod
+    def from_args(cls, _table_format_version: TableVersion = DEFAULT_READ_VERSION, **arguments: Any) -> ManifestEntry:
+        return super()._bind(**arguments, struct=MANIFEST_ENTRY_SCHEMAS_STRUCT[_table_format_version])
 
-    def __init__(self, *data: Any, **named_data: Any) -> None:
-        super().__init__(*data, **{"struct": MANIFEST_ENTRY_SCHEMAS_STRUCT[DEFAULT_READ_VERSION], **named_data})
+    @property
+    def status(self) -> ManifestEntryStatus:
+        return self._data[0]
 
-    def _wrap(
-        self,
-        new_status: ManifestEntryStatus,
-        new_snapshot_id: Optional[int],
-        new_sequence_number: Optional[int],
-        new_file_sequence_number: Optional[int],
-        new_file: DataFile,
-    ) -> ManifestEntry:
-        self.status = new_status
-        self.snapshot_id = new_snapshot_id
-        self.sequence_number = new_sequence_number
-        self.file_sequence_number = new_file_sequence_number
-        self.data_file = new_file
-        return self
+    @status.setter
+    def status(self, value: ManifestEntryStatus) -> None:
+        self._data[0] = value
 
-    def _wrap_append(
-        self, new_snapshot_id: Optional[int], new_sequence_number: Optional[int], new_file: DataFile
-    ) -> ManifestEntry:
-        return self._wrap(ManifestEntryStatus.ADDED, new_snapshot_id, new_sequence_number, None, new_file)
+    @property
+    def snapshot_id(self) -> Optional[int]:
+        return self._data[1]
 
-    def _wrap_delete(
-        self,
-        new_snapshot_id: Optional[int],
-        new_sequence_number: Optional[int],
-        new_file_sequence_number: Optional[int],
-        new_file: DataFile,
-    ) -> ManifestEntry:
-        return self._wrap(ManifestEntryStatus.DELETED, new_snapshot_id, new_sequence_number, new_file_sequence_number, new_file)
+    @snapshot_id.setter
+    def snapshot_id(self, value: int) -> None:
+        self._data[0] = value
 
-    def _wrap_existing(
-        self,
-        new_snapshot_id: Optional[int],
-        new_sequence_number: Optional[int],
-        new_file_sequence_number: Optional[int],
-        new_file: DataFile,
-    ) -> ManifestEntry:
-        return self._wrap(ManifestEntryStatus.EXISTING, new_snapshot_id, new_sequence_number, new_file_sequence_number, new_file)
+    @property
+    def sequence_number(self) -> Optional[int]:
+        return self._data[2]
+
+    @sequence_number.setter
+    def sequence_number(self, value: int) -> None:
+        self._data[2] = value
+
+    @property
+    def file_sequence_number(self) -> Optional[int]:
+        return self._data[3]
+
+    @file_sequence_number.setter
+    def file_sequence_number(self, value: int) -> None:
+        self._data[3] = value
+
+    @property
+    def data_file(self) -> DataFile:
+        return self._data[4]
+
+    @data_file.setter
+    def data_file(self, value: DataFile) -> None:
+        self._data[4] = value
 
 
 PARTITION_FIELD_SUMMARY_TYPE = StructType(
@@ -469,14 +634,25 @@ PARTITION_FIELD_SUMMARY_TYPE = StructType(
 
 
 class PartitionFieldSummary(Record):
-    __slots__ = ("contains_null", "contains_nan", "lower_bound", "upper_bound")
-    contains_null: bool
-    contains_nan: Optional[bool]
-    lower_bound: Optional[bytes]
-    upper_bound: Optional[bytes]
+    @classmethod
+    def from_args(cls, **arguments: Any) -> PartitionFieldSummary:
+        return super()._bind(**arguments, struct=PARTITION_FIELD_SUMMARY_TYPE)
 
-    def __init__(self, *data: Any, **named_data: Any) -> None:
-        super().__init__(*data, **{"struct": PARTITION_FIELD_SUMMARY_TYPE, **named_data})
+    @property
+    def contains_null(self) -> bool:
+        return self._data[0]
+
+    @property
+    def contains_nan(self) -> Optional[bool]:
+        return self._data[1]
+
+    @property
+    def lower_bound(self) -> Optional[bytes]:
+        return self._data[2]
+
+    @property
+    def upper_bound(self) -> Optional[bytes]:
+        return self._data[3]
 
 
 class PartitionFieldStats:
@@ -495,10 +671,10 @@ class PartitionFieldStats:
 
     def to_summary(self) -> PartitionFieldSummary:
         return PartitionFieldSummary(
-            contains_null=self._contains_null,
-            contains_nan=self._contains_nan,
-            lower_bound=to_bytes(self._type, self._min) if self._min is not None else None,
-            upper_bound=to_bytes(self._type, self._max) if self._max is not None else None,
+            self._contains_null,
+            self._contains_nan,
+            to_bytes(self._type, self._min) if self._min is not None else None,
+            to_bytes(self._type, self._max) if self._max is not None else None,
         )
 
     def update(self, value: Any) -> None:
@@ -559,6 +735,24 @@ MANIFEST_LIST_FILE_SCHEMAS: Dict[int, Schema] = {
         NestedField(507, "partitions", ListType(508, PARTITION_FIELD_SUMMARY_TYPE, element_required=True), required=False),
         NestedField(519, "key_metadata", BinaryType(), required=False),
     ),
+    3: Schema(
+        NestedField(500, "manifest_path", StringType(), required=True, doc="Location URI with FS scheme"),
+        NestedField(501, "manifest_length", LongType(), required=True),
+        NestedField(502, "partition_spec_id", IntegerType(), required=True),
+        NestedField(517, "content", IntegerType(), required=True, initial_default=ManifestContent.DATA),
+        NestedField(515, "sequence_number", LongType(), required=True, initial_default=0),
+        NestedField(516, "min_sequence_number", LongType(), required=True, initial_default=0),
+        NestedField(503, "added_snapshot_id", LongType(), required=True),
+        NestedField(504, "added_files_count", IntegerType(), required=True),
+        NestedField(505, "existing_files_count", IntegerType(), required=True),
+        NestedField(506, "deleted_files_count", IntegerType(), required=True),
+        NestedField(512, "added_rows_count", LongType(), required=True),
+        NestedField(513, "existing_rows_count", LongType(), required=True),
+        NestedField(514, "deleted_rows_count", LongType(), required=True),
+        NestedField(507, "partitions", ListType(508, PARTITION_FIELD_SUMMARY_TYPE, element_required=True), required=False),
+        NestedField(519, "key_metadata", BinaryType(), required=False),
+        NestedField(520, "first_row_id", LongType(), required=False),
+    ),
 }
 
 MANIFEST_LIST_FILE_STRUCTS = {format_version: schema.as_struct() for format_version, schema in MANIFEST_LIST_FILE_SCHEMAS.items()}
@@ -570,41 +764,77 @@ POSITIONAL_DELETE_SCHEMA = Schema(
 
 
 class ManifestFile(Record):
-    __slots__ = (
-        "manifest_path",
-        "manifest_length",
-        "partition_spec_id",
-        "content",
-        "sequence_number",
-        "min_sequence_number",
-        "added_snapshot_id",
-        "added_files_count",
-        "existing_files_count",
-        "deleted_files_count",
-        "added_rows_count",
-        "existing_rows_count",
-        "deleted_rows_count",
-        "partitions",
-        "key_metadata",
-    )
-    manifest_path: str
-    manifest_length: int
-    partition_spec_id: int
-    content: ManifestContent
-    sequence_number: int
-    min_sequence_number: int
-    added_snapshot_id: int
-    added_files_count: Optional[int]
-    existing_files_count: Optional[int]
-    deleted_files_count: Optional[int]
-    added_rows_count: Optional[int]
-    existing_rows_count: Optional[int]
-    deleted_rows_count: Optional[int]
-    partitions: Optional[List[PartitionFieldSummary]]
-    key_metadata: Optional[bytes]
+    @classmethod
+    def from_args(cls, _table_format_version: TableVersion = DEFAULT_READ_VERSION, **arguments: Any) -> ManifestFile:
+        return super()._bind(**arguments, struct=MANIFEST_LIST_FILE_SCHEMAS[_table_format_version])
 
-    def __init__(self, *data: Any, **named_data: Any) -> None:
-        super().__init__(*data, **{"struct": MANIFEST_LIST_FILE_STRUCTS[DEFAULT_READ_VERSION], **named_data})
+    @property
+    def manifest_path(self) -> str:
+        return self._data[0]
+
+    @property
+    def manifest_length(self) -> int:
+        return self._data[1]
+
+    @property
+    def partition_spec_id(self) -> int:
+        return self._data[2]
+
+    @property
+    def content(self) -> ManifestContent:
+        return self._data[3]
+
+    @property
+    def sequence_number(self) -> int:
+        return self._data[4]
+
+    @sequence_number.setter
+    def sequence_number(self, value: int) -> None:
+        self._data[4] = value
+
+    @property
+    def min_sequence_number(self) -> int:
+        return self._data[5]
+
+    @min_sequence_number.setter
+    def min_sequence_number(self, value: int) -> None:
+        self._data[5] = value
+
+    @property
+    def added_snapshot_id(self) -> Optional[int]:
+        return self._data[6]
+
+    @property
+    def added_files_count(self) -> Optional[int]:
+        return self._data[7]
+
+    @property
+    def existing_files_count(self) -> Optional[int]:
+        return self._data[8]
+
+    @property
+    def deleted_files_count(self) -> Optional[int]:
+        return self._data[9]
+
+    @property
+    def added_rows_count(self) -> Optional[int]:
+        return self._data[10]
+
+    @property
+    def existing_rows_count(self) -> Optional[int]:
+        return self._data[11]
+
+    @property
+    def deleted_rows_count(self) -> Optional[int]:
+        return self._data[12]
+
+    @property
+    def partitions(self) -> Optional[List[PartitionFieldSummary]]:
+        return self._data[13]
+
+    @property
+    def key_metadata(self) -> Optional[bytes]:
+        return self._data[14]
 
     def has_added_files(self) -> bool:
         return self.added_files_count is None or self.added_files_count > 0
@@ -636,6 +866,14 @@ class ManifestFile(Record):
                 if not discard_deleted or entry.status != ManifestEntryStatus.DELETED
             ]
 
+    def __eq__(self, other: Any) -> bool:
+        """Return the equality of two instances of the ManifestFile class."""
+        return self.manifest_path == other.manifest_path if isinstance(other, ManifestFile) else False
+
+    def __hash__(self) -> int:
+        """Return the hash of manifest_path."""
+        return hash(self.manifest_path)
+
 
 @cached(cache=LRUCache(maxsize=128), key=lambda io, manifest_list: hashkey(manifest_list))
 def _manifests(io: FileIO, manifest_list: str) -> Tuple[ManifestFile, ...]:
@@ -682,7 +920,7 @@ def _inherit_from_manifest(entry: ManifestEntry, manifest: ManifestFile) -> Mani
     """
     # Inherit sequence numbers.
     # The snapshot_id is required in V1, inherit with V2 when null
-    if entry.snapshot_id is None:
+    if entry.snapshot_id is None and manifest.added_snapshot_id is not None:
         entry.snapshot_id = manifest.added_snapshot_id
 
     # in v1 tables, the sequence number is not persisted and can be safely defaulted to 0
@@ -717,9 +955,16 @@ class ManifestWriter(ABC):
     _deleted_rows: int
     _min_sequence_number: Optional[int]
     _partitions: List[Record]
-    _reused_entry_wrapper: ManifestEntry
+    _compression: AvroCompressionCodec
 
-    def __init__(self, spec: PartitionSpec, schema: Schema, output_file: OutputFile, snapshot_id: int) -> None:
+    def __init__(
+        self,
+        spec: PartitionSpec,
+        schema: Schema,
+        output_file: OutputFile,
+        snapshot_id: int,
+        avro_compression: AvroCompressionCodec,
+    ) -> None:
         self.closed = False
         self._spec = spec
         self._schema = schema
@@ -734,7 +979,7 @@ class ManifestWriter(ABC):
         self._deleted_rows = 0
         self._min_sequence_number = None
         self._partitions = []
-        self._reused_entry_wrapper = ManifestEntry()
+        self._compression = avro_compression
 
     def __enter__(self) -> ManifestWriter:
         """Open the writer."""
@@ -770,6 +1015,7 @@ class ManifestWriter(ABC):
             "partition-spec": to_json(self._spec.fields).decode("utf-8"),
             "partition-spec-id": str(self._spec.spec_id),
             "format-version": str(self.version),
+            AVRO_CODEC_KEY: self._compression,
         }
 
     def _with_partition(self, format_version: TableVersion) -> Schema:
@@ -795,7 +1041,7 @@ class ManifestWriter(ABC):
         # once the manifest file is generated, no more entries can be added
         self.closed = True
         min_sequence_number = self._min_sequence_number or UNASSIGNED_SEQ
-        return ManifestFile(
+        return ManifestFile.from_args(
             manifest_path=self._output_file.location,
             manifest_length=len(self._writer.output_file),
             partition_spec_id=self._spec.spec_id,
@@ -841,37 +1087,52 @@ class ManifestWriter(ABC):
         return self
 
     def add(self, entry: ManifestEntry) -> ManifestWriter:
-        if entry.sequence_number is not None and entry.sequence_number >= 0:
-            self.add_entry(self._reused_entry_wrapper._wrap_append(self._snapshot_id, entry.sequence_number, entry.data_file))
-        else:
-            self.add_entry(self._reused_entry_wrapper._wrap_append(self._snapshot_id, None, entry.data_file))
+        self.add_entry(
+            ManifestEntry.from_args(
+                status=ManifestEntryStatus.ADDED,
+                snapshot_id=self._snapshot_id,
+                sequence_number=entry.sequence_number if entry.sequence_number != UNASSIGNED_SEQ else None,
+                data_file=entry.data_file,
+            )
+        )
+
         return self
 
     def delete(self, entry: ManifestEntry) -> ManifestWriter:
         self.add_entry(
-            self._reused_entry_wrapper._wrap_delete(
-                self._snapshot_id, entry.sequence_number, entry.file_sequence_number, entry.data_file
+            ManifestEntry.from_args(
+                status=ManifestEntryStatus.DELETED,
+                snapshot_id=self._snapshot_id,
+                sequence_number=entry.sequence_number,
+                file_sequence_number=entry.file_sequence_number,
+                data_file=entry.data_file,
             )
         )
         return self
 
     def existing(self, entry: ManifestEntry) -> ManifestWriter:
         self.add_entry(
-            self._reused_entry_wrapper._wrap_existing(
-                entry.snapshot_id, entry.sequence_number, entry.file_sequence_number, entry.data_file
+            ManifestEntry.from_args(
+                status=ManifestEntryStatus.EXISTING,
+                snapshot_id=entry.snapshot_id,
+                sequence_number=entry.sequence_number,
+                file_sequence_number=entry.file_sequence_number,
+                data_file=entry.data_file,
             )
         )
         return self
 
 
 class ManifestWriterV1(ManifestWriter):
-    def __init__(self, spec: PartitionSpec, schema: Schema, output_file: OutputFile, snapshot_id: int):
-        super().__init__(
-            spec,
-            schema,
-            output_file,
-            snapshot_id,
-        )
+    def __init__(
+        self,
+        spec: PartitionSpec,
+        schema: Schema,
+        output_file: OutputFile,
+        snapshot_id: int,
+        avro_compression: AvroCompressionCodec,
+    ):
+        super().__init__(spec, schema, output_file, snapshot_id, avro_compression)
 
     def content(self) -> ManifestContent:
         return ManifestContent.DATA
@@ -885,8 +1146,15 @@ class ManifestWriterV1(ManifestWriter):
 
 
 class ManifestWriterV2(ManifestWriter):
-    def __init__(self, spec: PartitionSpec, schema: Schema, output_file: OutputFile, snapshot_id: int):
-        super().__init__(spec, schema, output_file, snapshot_id)
+    def __init__(
+        self,
+        spec: PartitionSpec,
+        schema: Schema,
+        output_file: OutputFile,
+        snapshot_id: int,
+        avro_compression: AvroCompressionCodec,
+    ):
+        super().__init__(spec, schema, output_file, snapshot_id, avro_compression)
 
     def content(self) -> ManifestContent:
         return ManifestContent.DATA
@@ -912,12 +1180,17 @@ class ManifestWriterV2(ManifestWriter):
 
 
 def write_manifest(
-    format_version: TableVersion, spec: PartitionSpec, schema: Schema, output_file: OutputFile, snapshot_id: int
+    format_version: TableVersion,
+    spec: PartitionSpec,
+    schema: Schema,
+    output_file: OutputFile,
+    snapshot_id: int,
+    avro_compression: AvroCompressionCodec,
 ) -> ManifestWriter:
     if format_version == 1:
-        return ManifestWriterV1(spec, schema, output_file, snapshot_id)
+        return ManifestWriterV1(spec, schema, output_file, snapshot_id, avro_compression)
     elif format_version == 2:
-        return ManifestWriterV2(spec, schema, output_file, snapshot_id)
+        return ManifestWriterV2(spec, schema, output_file, snapshot_id, avro_compression)
     else:
         raise ValueError(f"Cannot write manifest for table version: {format_version}")
 
@@ -967,7 +1240,13 @@ class ManifestListWriter(ABC):
 
 
 class ManifestListWriterV1(ManifestListWriter):
-    def __init__(self, output_file: OutputFile, snapshot_id: int, parent_snapshot_id: Optional[int]):
+    def __init__(
+        self,
+        output_file: OutputFile,
+        snapshot_id: int,
+        parent_snapshot_id: Optional[int],
+        compression: AvroCompressionCodec,
+    ):
         super().__init__(
             format_version=1,
             output_file=output_file,
@@ -975,6 +1254,7 @@ class ManifestListWriterV1(ManifestListWriter):
                 "snapshot-id": str(snapshot_id),
                 "parent-snapshot-id": str(parent_snapshot_id) if parent_snapshot_id is not None else "null",
                 "format-version": "1",
+                AVRO_CODEC_KEY: compression,
             },
         )
 
@@ -988,7 +1268,14 @@ class ManifestListWriterV2(ManifestListWriter):
     _commit_snapshot_id: int
     _sequence_number: int
 
-    def __init__(self, output_file: OutputFile, snapshot_id: int, parent_snapshot_id: Optional[int], sequence_number: int):
+    def __init__(
+        self,
+        output_file: OutputFile,
+        snapshot_id: int,
+        parent_snapshot_id: Optional[int],
+        sequence_number: int,
+        compression: AvroCompressionCodec,
+    ):
         super().__init__(
             format_version=2,
             output_file=output_file,
@@ -997,6 +1284,7 @@ class ManifestListWriterV2(ManifestListWriter):
                 "parent-snapshot-id": str(parent_snapshot_id) if parent_snapshot_id is not None else "null",
                 "sequence-number": str(sequence_number),
                 "format-version": "2",
+                AVRO_CODEC_KEY: compression,
             },
         )
         self._commit_snapshot_id = snapshot_id
@@ -1031,12 +1319,13 @@ def write_manifest_list(
     snapshot_id: int,
     parent_snapshot_id: Optional[int],
     sequence_number: Optional[int],
+    avro_compression: AvroCompressionCodec,
 ) -> ManifestListWriter:
     if format_version == 1:
-        return ManifestListWriterV1(output_file, snapshot_id, parent_snapshot_id)
+        return ManifestListWriterV1(output_file, snapshot_id, parent_snapshot_id, avro_compression)
     elif format_version == 2:
         if sequence_number is None:
             raise ValueError(f"Sequence-number is required for V2 tables: {sequence_number}")
-        return ManifestListWriterV2(output_file, snapshot_id, parent_snapshot_id, sequence_number)
+        return ManifestListWriterV2(output_file, snapshot_id, parent_snapshot_id, sequence_number, avro_compression)
     else:
         raise ValueError(f"Cannot write manifest list for table version: {format_version}")
