@@ -35,6 +35,7 @@ from __future__ import annotations
 import re
 from functools import cached_property
 from typing import (
+    Annotated,
     Any,
     ClassVar,
     Dict,
@@ -44,16 +45,18 @@ from typing import (
 )
 
 from pydantic import (
+    BeforeValidator,
     Field,
     PrivateAttr,
     SerializeAsAny,
+    field_validator,
     model_serializer,
     model_validator,
 )
-from pydantic_core.core_schema import ValidatorFunctionWrapHandler
+from pydantic_core.core_schema import ValidationInfo, ValidatorFunctionWrapHandler
 
 from pyiceberg.exceptions import ValidationError
-from pyiceberg.typedef import IcebergBaseModel, IcebergRootModel, L
+from pyiceberg.typedef import IcebergBaseModel, IcebergRootModel, L, TableVersion
 from pyiceberg.utils.parsing import ParseNumberFromBrackets
 from pyiceberg.utils.singleton import Singleton
 
@@ -140,6 +143,10 @@ class IcebergType(IcebergBaseModel):
                 return TimestampType()
             if v == "timestamptz":
                 return TimestamptzType()
+            if v == "timestamp_ns":
+                return TimestampNanoType()
+            if v == "timestamptz_ns":
+                return TimestamptzNanoType()
             if v == "date":
                 return DateType()
             if v == "time":
@@ -148,13 +155,15 @@ class IcebergType(IcebergBaseModel):
                 return UUIDType()
             if v == "binary":
                 return BinaryType()
+            if v == "unknown":
+                return UnknownType()
             if v.startswith("fixed"):
                 return FixedType(_parse_fixed_type(v))
             if v.startswith("decimal"):
                 precision, scale = _parse_decimal_type(v)
                 return DecimalType(precision, scale)
             else:
-                raise ValueError(f"Unknown type: {v}")
+                raise ValueError(f"Type not recognized: {v}")
         if isinstance(v, dict) and cls == IcebergType:
             complex_type = v.get("type")
             if complex_type == "list":
@@ -175,6 +184,10 @@ class IcebergType(IcebergBaseModel):
     def is_struct(self) -> bool:
         return isinstance(self, StructType)
 
+    def minimum_format_version(self) -> TableVersion:
+        """Minimum Iceberg format version after which this type is supported."""
+        return 1
+
 
 class PrimitiveType(Singleton, IcebergRootModel[str], IcebergType):
     """Base class for all Iceberg Primitive Types."""
@@ -279,6 +292,18 @@ class DecimalType(PrimitiveType):
         return self.root == other.root if isinstance(other, DecimalType) else False
 
 
+def _deserialize_default_value(v: Any, context: ValidationInfo) -> Any:
+    if v is not None:
+        from pyiceberg.conversions import from_json
+
+        return from_json(context.data.get("field_type"), v)
+    else:
+        return None
+
+
+DefaultValue = Annotated[L, BeforeValidator(_deserialize_default_value)]
+
+
 class NestedField(IcebergType):
     """Represents a field of a struct, a map key, a map value, or a list element.
 
@@ -300,6 +325,14 @@ class NestedField(IcebergType):
         ...     doc="Just a long"
         ... ))
         '2: bar: required long (Just a long)'
+        >>> str(NestedField(
+        ...     field_id=3,
+        ...     name='baz',
+        ...     field_type="string",
+        ...     required=True,
+        ...     doc="A string field"
+        ... ))
+        '3: baz: required string (A string field)'
     """
 
     field_id: int = Field(alias="id")
@@ -307,14 +340,24 @@ class NestedField(IcebergType):
     field_type: SerializeAsAny[IcebergType] = Field(alias="type")
     required: bool = Field(default=False)
     doc: Optional[str] = Field(default=None, repr=False)
-    initial_default: Optional[Any] = Field(alias="initial-default", default=None, repr=False)
-    write_default: Optional[L] = Field(alias="write-default", default=None, repr=False)  # type: ignore
+    initial_default: Optional[DefaultValue] = Field(alias="initial-default", default=None, repr=True)  # type: ignore
+    write_default: Optional[DefaultValue] = Field(alias="write-default", default=None, repr=True)  # type: ignore
+
+    @field_validator("field_type", mode="before")
+    def convert_field_type(cls, v: Any) -> IcebergType:
+        """Convert string values into IcebergType instances."""
+        if isinstance(v, str):
+            try:
+                return IcebergType.handle_primitive_type(v, None)
+            except ValueError as e:
+                raise ValueError(f"Unsupported field type: '{v}'") from e
+        return v
 
     def __init__(
         self,
         field_id: Optional[int] = None,
         name: Optional[str] = None,
-        field_type: Optional[IcebergType] = None,
+        field_type: Optional[IcebergType | str] = None,
         required: bool = False,
         doc: Optional[str] = None,
         initial_default: Optional[Any] = None,
@@ -332,12 +375,47 @@ class NestedField(IcebergType):
         data["write-default"] = data["write-default"] if "write-default" in data else write_default
         super().__init__(**data)
 
+    @model_serializer()
+    def serialize_model(self) -> Dict[str, Any]:
+        from pyiceberg.conversions import to_json
+
+        fields = {
+            "id": self.field_id,
+            "name": self.name,
+            "type": self.field_type,
+            "required": self.required,
+        }
+
+        if self.doc is not None:
+            fields["doc"] = self.doc
+        if self.initial_default is not None:
+            fields["initial-default"] = to_json(self.field_type, self.initial_default)
+        if self.write_default is not None:
+            fields["write-default"] = to_json(self.field_type, self.write_default)
+
+        return fields
+
     def __str__(self) -> str:
         """Return the string representation of the NestedField class."""
         doc = "" if not self.doc else f" ({self.doc})"
         req = "required" if self.required else "optional"
         return f"{self.field_id}: {self.name}: {req} {self.field_type}{doc}"
 
+    def __repr__(self) -> str:
+        """Return the string representation of the NestedField class."""
+        parts = [
+            f"field_id={self.field_id}",
+            f"name={self.name!r}",
+            f"field_type={self.field_type!r}",
+            f"required={self.required}",
+        ]
+        if self.initial_default is not None:
+            parts.append(f"initial_default={self.initial_default!r}")
+        if self.write_default is not None:
+            parts.append(f"write_default={self.write_default!r}")
+
+        return f"NestedField({', '.join(parts)})"
+
     def __getnewargs__(self) -> Tuple[int, str, IcebergType, bool, Optional[str]]:
         """Pickle the NestedField class."""
         return (self.field_id, self.name, self.field_type, self.required, self.doc)
@@ -701,6 +779,44 @@ class TimestamptzType(PrimitiveType):
     root: Literal["timestamptz"] = Field(default="timestamptz")
 
 
+class TimestampNanoType(PrimitiveType):
+    """A TimestampNano data type in Iceberg can be represented using an instance of this class.
+
+    TimestampNanos in Iceberg have nanosecond precision and include a date and a time of day without a timezone.
+
+    Example:
+        >>> column_foo = TimestampNanoType()
+        >>> isinstance(column_foo, TimestampNanoType)
+        True
+        >>> column_foo
+        TimestampNanoType()
+    """
+
+    root: Literal["timestamp_ns"] = Field(default="timestamp_ns")
+
+    def minimum_format_version(self) -> TableVersion:
+        return 3
+
+
+class TimestamptzNanoType(PrimitiveType):
+    """A TimestamptzNano data type in Iceberg can be represented using an instance of this class.
+
+    TimestamptzNanos in Iceberg are stored as UTC and include a date and a time of day with a timezone.
+
+    Example:
+        >>> column_foo = TimestamptzNanoType()
+        >>> isinstance(column_foo, TimestamptzNanoType)
+        True
+        >>> column_foo
+        TimestamptzNanoType()
+    """
+
+    root: Literal["timestamptz_ns"] = Field(default="timestamptz_ns")
+
+    def minimum_format_version(self) -> TableVersion:
+        return 3
+
+
 class StringType(PrimitiveType):
     """A String data type in Iceberg can be represented using an instance of this class.
 
@@ -747,3 +863,22 @@ class BinaryType(PrimitiveType):
     """
 
     root: Literal["binary"] = Field(default="binary")
+
+
+class UnknownType(PrimitiveType):
+    """An unknown data type in Iceberg can be represented using an instance of this class.
+
+    Unknowns in Iceberg are used to represent data types that are not known at the time of writing.
+
+    Example:
+        >>> column_foo = UnknownType()
+        >>> isinstance(column_foo, UnknownType)
+        True
+        >>> column_foo
+        UnknownType()
+    """
+
+    root: Literal["unknown"] = Field(default="unknown")
+
+    def minimum_format_version(self) -> TableVersion:
+        return 3
