@@ -64,8 +64,6 @@ from pyiceberg.expressions.visitors import _from_byte_buffer
 from pyiceberg.schema import Accessor, Schema
 from pyiceberg.typedef import Record
 from pyiceberg.types import (
-    BinaryType,
-    BooleanType,
     DecimalType,
     DoubleType,
     FloatType,
@@ -75,7 +73,6 @@ from pyiceberg.types import (
     NestedField,
     StringType,
     StructType,
-    UUIDType,
 )
 from pyiceberg.utils.singleton import Singleton
 
@@ -630,22 +627,7 @@ def test_invert_always() -> None:
 def test_accessor_base_class() -> None:
     """Test retrieving a value at a position of a container using an accessor"""
 
-    struct = Record(
-        struct=StructType(
-            NestedField(1, "a", StringType()),
-            NestedField(2, "b", StringType()),
-            NestedField(3, "c", StringType()),
-            NestedField(4, "d", IntegerType()),
-            NestedField(5, "e", IntegerType()),
-            NestedField(6, "f", IntegerType()),
-            NestedField(7, "g", FloatType()),
-            NestedField(8, "h", DecimalType(8, 4)),
-            NestedField(9, "i", UUIDType()),
-            NestedField(10, "j", BooleanType()),
-            NestedField(11, "k", BooleanType()),
-            NestedField(12, "l", BinaryType()),
-        )
-    )
+    struct = Record(*[None] * 12)
 
     uuid_value = uuid.uuid4()
 
@@ -985,11 +967,7 @@ def test_less_than_or_equal() -> None:
 
 def test_bound_reference_eval(table_schema_simple: Schema) -> None:
     """Test creating a BoundReference and evaluating it on a StructProtocol"""
-    struct = Record(struct=table_schema_simple.as_struct())
-
-    struct[0] = "foovalue"
-    struct[1] = 123
-    struct[2] = True
+    struct = Record("foovalue", 123, True)
 
     position1_accessor = Accessor(position=0)
     position2_accessor = Accessor(position=1)
