@@ -195,7 +195,7 @@ def test_record_count() -> None:
         stats_columns=compute_statistics_plan(schema, table_metadata.properties),
         parquet_column_mapping=parquet_path_to_id_mapping(schema),
     )
-    datafile = DataFile(**statistics.to_serialized_dict())
+    datafile = DataFile.from_args(**statistics.to_serialized_dict())
     assert datafile.record_count == 4
 
 
@@ -208,7 +208,7 @@ def test_value_counts() -> None:
         stats_columns=compute_statistics_plan(schema, table_metadata.properties),
         parquet_column_mapping=parquet_path_to_id_mapping(schema),
     )
-    datafile = DataFile(**statistics.to_serialized_dict())
+    datafile = DataFile.from_args(**statistics.to_serialized_dict())
 
     assert len(datafile.value_counts) == 7
     assert datafile.value_counts[1] == 4
@@ -229,7 +229,7 @@ def test_column_sizes() -> None:
         stats_columns=compute_statistics_plan(schema, table_metadata.properties),
         parquet_column_mapping=parquet_path_to_id_mapping(schema),
     )
-    datafile = DataFile(**statistics.to_serialized_dict())
+    datafile = DataFile.from_args(**statistics.to_serialized_dict())
 
     assert len(datafile.column_sizes) == 7
     # these values are an artifact of how the write_table encodes the columns
@@ -249,7 +249,7 @@ def test_null_and_nan_counts() -> None:
         stats_columns=compute_statistics_plan(schema, table_metadata.properties),
         parquet_column_mapping=parquet_path_to_id_mapping(schema),
     )
-    datafile = DataFile(**statistics.to_serialized_dict())
+    datafile = DataFile.from_args(**statistics.to_serialized_dict())
 
     assert len(datafile.null_value_counts) == 7
     assert datafile.null_value_counts[1] == 1
@@ -276,7 +276,7 @@ def test_bounds() -> None:
         stats_columns=compute_statistics_plan(schema, table_metadata.properties),
         parquet_column_mapping=parquet_path_to_id_mapping(schema),
     )
-    datafile = DataFile(**statistics.to_serialized_dict())
+    datafile = DataFile.from_args(**statistics.to_serialized_dict())
 
     assert len(datafile.lower_bounds) == 2
     assert datafile.lower_bounds[1].decode() == "aaaaaaaaaaaaaaaa"
@@ -320,7 +320,7 @@ def test_metrics_mode_none() -> None:
         stats_columns=compute_statistics_plan(schema, table_metadata.properties),
         parquet_column_mapping=parquet_path_to_id_mapping(schema),
     )
-    datafile = DataFile(**statistics.to_serialized_dict())
+    datafile = DataFile.from_args(**statistics.to_serialized_dict())
 
     assert len(datafile.value_counts) == 0
     assert len(datafile.null_value_counts) == 0
@@ -339,7 +339,7 @@ def test_metrics_mode_counts() -> None:
         stats_columns=compute_statistics_plan(schema, table_metadata.properties),
         parquet_column_mapping=parquet_path_to_id_mapping(schema),
     )
-    datafile = DataFile(**statistics.to_serialized_dict())
+    datafile = DataFile.from_args(**statistics.to_serialized_dict())
 
     assert len(datafile.value_counts) == 7
     assert len(datafile.null_value_counts) == 7
@@ -358,7 +358,7 @@ def test_metrics_mode_full() -> None:
         stats_columns=compute_statistics_plan(schema, table_metadata.properties),
         parquet_column_mapping=parquet_path_to_id_mapping(schema),
     )
-    datafile = DataFile(**statistics.to_serialized_dict())
+    datafile = DataFile.from_args(**statistics.to_serialized_dict())
 
     assert len(datafile.value_counts) == 7
     assert len(datafile.null_value_counts) == 7
@@ -383,7 +383,7 @@ def test_metrics_mode_non_default_trunc() -> None:
         stats_columns=compute_statistics_plan(schema, table_metadata.properties),
         parquet_column_mapping=parquet_path_to_id_mapping(schema),
     )
-    datafile = DataFile(**statistics.to_serialized_dict())
+    datafile = DataFile.from_args(**statistics.to_serialized_dict())
 
     assert len(datafile.value_counts) == 7
     assert len(datafile.null_value_counts) == 7
@@ -409,7 +409,7 @@ def test_column_metrics_mode() -> None:
         stats_columns=compute_statistics_plan(schema, table_metadata.properties),
         parquet_column_mapping=parquet_path_to_id_mapping(schema),
     )
-    datafile = DataFile(**statistics.to_serialized_dict())
+    datafile = DataFile.from_args(**statistics.to_serialized_dict())
 
     assert len(datafile.value_counts) == 6
     assert len(datafile.null_value_counts) == 6
@@ -450,6 +450,9 @@ def construct_test_table_primitive_types() -> Tuple[pq.FileMetaData, Union[Table
                     {"id": 13, "name": "decimal8", "required": False, "type": "decimal(5, 2)"},
                     {"id": 14, "name": "decimal16", "required": False, "type": "decimal(16, 6)"},
                     {"id": 15, "name": "decimal32", "required": False, "type": "decimal(19, 6)"},
+                    {"id": 16, "name": "empty_decimal8", "required": False, "type": "decimal(5, 2)"},
+                    {"id": 17, "name": "empty_decimal16", "required": False, "type": "decimal(16, 6)"},
+                    {"id": 18, "name": "empty_decimal32", "required": False, "type": "decimal(19, 6)"},
                 ],
             },
         ],
@@ -477,6 +480,9 @@ def construct_test_table_primitive_types() -> Tuple[pq.FileMetaData, Union[Table
     decimal8 = pa.array([Decimal("123.45"), Decimal("678.91")], pa.decimal128(8, 2))
     decimal16 = pa.array([Decimal("12345679.123456"), Decimal("67891234.678912")], pa.decimal128(16, 6))
     decimal32 = pa.array([Decimal("1234567890123.123456"), Decimal("9876543210703.654321")], pa.decimal128(19, 6))
+    empty_decimal8 = pa.array([None, None], pa.decimal128(8, 2))
+    empty_decimal16 = pa.array([None, None], pa.decimal128(16, 6))
+    empty_decimal32 = pa.array([None, None], pa.decimal128(19, 6))
 
     table = pa.Table.from_pydict(
         {
@@ -495,6 +501,9 @@ def construct_test_table_primitive_types() -> Tuple[pq.FileMetaData, Union[Table
             "decimal8": decimal8,
             "decimal16": decimal16,
             "decimal32": decimal32,
+            "empty_decimal8": empty_decimal8,
+            "empty_decimal16": empty_decimal16,
+            "empty_decimal32": empty_decimal32,
         },
         schema=arrow_schema,
     )
@@ -518,10 +527,16 @@ def test_metrics_primitive_types() -> None:
         stats_columns=compute_statistics_plan(schema, table_metadata.properties),
         parquet_column_mapping=parquet_path_to_id_mapping(schema),
     )
-    datafile = DataFile(**statistics.to_serialized_dict())
+    datafile = DataFile.from_args(**statistics.to_serialized_dict())
 
-    assert len(datafile.value_counts) == 15
-    assert len(datafile.null_value_counts) == 15
+    assert len(datafile.value_counts) == 18
+    assert len(datafile.null_value_counts) == 18
+    assert datafile.null_value_counts[13] != datafile.value_counts[13]
+    assert datafile.null_value_counts[14] != datafile.value_counts[14]
+    assert datafile.null_value_counts[15] != datafile.value_counts[15]
+    assert datafile.null_value_counts[16] == datafile.value_counts[16]
+    assert datafile.null_value_counts[17] == datafile.value_counts[17]
+    assert datafile.null_value_counts[18] == datafile.value_counts[18]
     assert len(datafile.nan_value_counts) == 0
 
     tz = timezone(timedelta(seconds=19800))
@@ -542,6 +557,7 @@ def test_metrics_primitive_types() -> None:
     assert datafile.lower_bounds[13][::-1].ljust(4, b"\x00") == STRUCT_INT32.pack(12345)
     assert datafile.lower_bounds[14][::-1].ljust(8, b"\x00") == STRUCT_INT64.pack(12345679123456)
     assert str(int.from_bytes(datafile.lower_bounds[15], byteorder="big", signed=True)).encode("utf-8") == b"1234567890123123456"
+    assert not any(key in datafile.lower_bounds.keys() for key in [16, 17, 18])
 
     assert len(datafile.upper_bounds) == 15
     assert datafile.upper_bounds[1] == STRUCT_BOOL.pack(True)
@@ -559,6 +575,7 @@ def test_metrics_primitive_types() -> None:
     assert datafile.upper_bounds[13][::-1].ljust(4, b"\x00") == STRUCT_INT32.pack(67891)
     assert datafile.upper_bounds[14][::-1].ljust(8, b"\x00") == STRUCT_INT64.pack(67891234678912)
     assert str(int.from_bytes(datafile.upper_bounds[15], byteorder="big", signed=True)).encode("utf-8") == b"9876543210703654321"
+    assert not any(key in datafile.upper_bounds.keys() for key in [16, 17, 18])
 
 
 def construct_test_table_invalid_upper_bound() -> Tuple[pq.FileMetaData, Union[TableMetadataV1, TableMetadataV2]]:
@@ -622,7 +639,7 @@ def test_metrics_invalid_upper_bound() -> None:
         stats_columns=compute_statistics_plan(schema, table_metadata.properties),
         parquet_column_mapping=parquet_path_to_id_mapping(schema),
     )
-    datafile = DataFile(**statistics.to_serialized_dict())
+    datafile = DataFile.from_args(**statistics.to_serialized_dict())
 
     assert len(datafile.value_counts) == 4
     assert len(datafile.null_value_counts) == 4
@@ -648,7 +665,7 @@ def test_offsets() -> None:
         stats_columns=compute_statistics_plan(schema, table_metadata.properties),
         parquet_column_mapping=parquet_path_to_id_mapping(schema),
     )
-    datafile = DataFile(**statistics.to_serialized_dict())
+    datafile = DataFile.from_args(**statistics.to_serialized_dict())
 
     assert datafile.split_offsets is not None
     assert len(datafile.split_offsets) == 1
@@ -723,7 +740,7 @@ def test_read_missing_statistics() -> None:
         parquet_column_mapping=parquet_path_to_id_mapping(schema),
     )
 
-    datafile = DataFile(**statistics.to_serialized_dict())
+    datafile = DataFile.from_args(**statistics.to_serialized_dict())
 
     # expect only "strings" column values to be reflected in the
     # upper_bound, lower_bound and null_value_counts props of datafile
