@@ -43,6 +43,10 @@ Notes:
         - Stored as microseconds from 1970-01-01 00:00:00.000000 in an 8-byte little-endian long
         - 400000L is 0...110|00011010|10000000 in binary
         - 10000000 -> 128 (-128), 00011010 -> 26, 00000110 -> 6, ... , 00000000 -> 0
+    TimestampNano:
+        - Stored as nanoseconds from 1970-01-01 00:00:00.000000000 in an 8-byte little-endian long
+        - 400000000L is 00010111|11010111|10000100|00000000 in binary
+        - 00000000 -> 0, 10000100 -> 124 (-124), 11010111 -> 41 (-41), 00010111 -> 23, ... , 00000000 -> 0
     String:
         - Stored as UTF-8 bytes (without length)
         - 'A' -> 65, 'B' -> 66, 'C' -> 67
@@ -99,7 +103,9 @@ from pyiceberg.types import (
     LongType,
     PrimitiveType,
     StringType,
+    TimestampNanoType,
     TimestampType,
+    TimestamptzNanoType,
     TimestamptzType,
     TimeType,
     UUIDType,
@@ -266,6 +272,8 @@ def test_partition_to_py_raise_on_incorrect_precision_or_scale(
         (TimestamptzType(), b"\x00\xe8vH\x17\x00\x00\x00", 100000000000),
         (TimestampType(), b"\x80\x1a\x06\x00\x00\x00\x00\x00", 400000),
         (TimestampType(), b"\x00\xe8vH\x17\x00\x00\x00", 100000000000),
+        (TimestampNanoType(), b"\00\x84\xd7\x17\x00\x00\x00\x00", 400000000),
+        (TimestamptzNanoType(), b"\00\x84\xd7\x17\x00\x00\x00\x00", 400000000),
         (StringType(), b"ABC", "ABC"),
         (StringType(), b"foo", "foo"),
         (
@@ -545,3 +553,52 @@ def test_datetime_obj_to_bytes(primitive_type: PrimitiveType, value: Union[datet
     bytes_from_value = conversions.to_bytes(primitive_type, value)
 
     assert bytes_from_value == expected_bytes
+
+
+@pytest.mark.parametrize(
+    "primitive_type, value, expected",
+    [
+        (BooleanType(), True, True),
+        (IntegerType(), 34, 34),
+        (LongType(), 34, 34),
+        (FloatType(), 1.0, 1.0),
+        (DoubleType(), 1.0, 1.0),
+        (DecimalType(9, 4), Decimal("123.4500"), "123.4500"),
+        (DecimalType(9, 0), Decimal("2"), "2"),
+        (DecimalType(9, -20), Decimal("2E+20"), "2E+20"),
+        (DateType(), date(2017, 11, 16), "2017-11-16"),
+        (TimeType(), time(22, 31, 8, 123456), "22:31:08.123456"),
+        (TimestampType(), datetime(2017, 11, 16, 22, 31, 8, 123456), "2017-11-16T22:31:08.123456"),
+        (TimestamptzType(), datetime(2017, 11, 16, 22, 31, 8, 123456, tzinfo=timezone.utc), "2017-11-16T22:31:08.123456+00:00"),
+        (StringType(), "iceberg", "iceberg"),
+        (BinaryType(), b"\x01\x02\x03\xff", "010203ff"),
+        (FixedType(4), b"\x01\x02\x03\xff", "010203ff"),
+    ],
+)
+def test_json_single_serialization(primitive_type: PrimitiveType, value: Any, expected: Any) -> None:
+    json_val = conversions.to_json(primitive_type, value)
+    assert json_val == expected
+
+
+@pytest.mark.parametrize(
+    "primitive_type, value",
+    [
+        (BooleanType(), True),
+        (IntegerType(), 34),
+        (LongType(), 34),
+        (FloatType(), 1.0),
+        (DoubleType(), 1.0),
+        (DecimalType(9, 4), Decimal("123.4500")),
+        (DecimalType(9, 0), Decimal("2")),
+        (DecimalType(9, -20), Decimal("2E+20")),
+        (DateType(), date(2017, 11, 16)),
+        (TimeType(), time(22, 31, 8, 123456)),
+        (TimestampType(), datetime(2017, 11, 16, 22, 31, 8, 123456)),
+        (TimestamptzType(), datetime(2017, 11, 16, 22, 31, 8, 123456, tzinfo=timezone.utc)),
+        (StringType(), "iceberg"),
+        (BinaryType(), b"\x01\x02\x03\xff"),
+        (FixedType(4), b"\x01\x02\x03\xff"),
+    ],
+)
+def test_json_serialize_roundtrip(primitive_type: PrimitiveType, value: Any) -> None:
+    assert value == conversions.from_json(primitive_type, conversions.to_json(primitive_type, value))
