@@ -22,6 +22,7 @@ from unittest.mock import patch
 import fastavro
 import pytest
 
+from pyiceberg.avro.codecs import AvroCompressionCodec
 from pyiceberg.io import load_file_io
 from pyiceberg.io.pyarrow import PyArrowFileIO
 from pyiceberg.manifest import (
@@ -37,10 +38,9 @@ from pyiceberg.manifest import (
     write_manifest,
     write_manifest_list,
 )
-from pyiceberg.partitioning import UNPARTITIONED_PARTITION_SPEC, PartitionField, PartitionSpec
+from pyiceberg.partitioning import UNPARTITIONED_PARTITION_SPEC, PartitionSpec
 from pyiceberg.schema import Schema
 from pyiceberg.table.snapshots import Operation, Snapshot, Summary
-from pyiceberg.transforms import IdentityTransform
 from pyiceberg.typedef import Record, TableVersion
 from pyiceberg.types import IntegerType, NestedField
 
@@ -61,7 +61,7 @@ def _verify_metadata_with_fastavro(avro_file: str, expected_metadata: Dict[str,
 
 
 def test_read_manifest_entry(generated_manifest_entry_file: str) -> None:
-    manifest = ManifestFile(
+    manifest = ManifestFile.from_args(
         manifest_path=generated_manifest_entry_file,
         manifest_length=0,
         partition_spec_id=0,
@@ -79,13 +79,13 @@ def test_read_manifest_entry(generated_manifest_entry_file: str) -> None:
 
     data_file = manifest_entry.data_file
 
-    assert data_file.content is DataFileContent.DATA
+    assert data_file.content == DataFileContent.DATA
     assert (
         data_file.file_path
         == "/home/iceberg/warehouse/nyc/taxis_partitioned/data/VendorID=null/00000-633-d8a4223e-dc97-45a1-86e1-adaba6e8abd7-00001.parquet"
     )
     assert data_file.file_format == FileFormat.PARQUET
-    assert repr(data_file.partition) == "Record[VendorID=1, tpep_pickup_datetime=1925]"
+    assert repr(data_file.partition) == "Record[1, 1925]"
     assert data_file.record_count == 19513
     assert data_file.file_size_in_bytes == 388872
     assert data_file.column_sizes == {
@@ -153,8 +153,8 @@ def test_read_manifest_entry(generated_manifest_entry_file: str) -> None:
     }
     assert data_file.nan_value_counts == {16: 0, 17: 0, 18: 0, 19: 0, 10: 0, 11: 0, 12: 0, 13: 0, 14: 0, 15: 0}
     assert data_file.lower_bounds == {
-        2: b"2020-04-01 00:00",
-        3: b"2020-04-01 00:12",
+        2: b"\x01\x00\x00\x00\x00\x00\x00\x00",
+        3: b"\x01\x00\x00\x00\x00\x00\x00\x00",
         7: b"\x03\x00\x00\x00",
         8: b"\x01\x00\x00\x00",
         10: b"\xf6(\\\x8f\xc2\x05S\xc0",
@@ -168,8 +168,8 @@ def test_read_manifest_entry(generated_manifest_entry_file: str) -> None:
         19: b"\x00\x00\x00\x00\x00\x00\x04\xc0",
     }
     assert data_file.upper_bounds == {
-        2: b"2020-04-30 23:5:",
-        3: b"2020-05-01 00:41",
+        2: b"\x06\x00\x00\x00\x00\x00\x00\x00",
+        3: b"\x06\x00\x00\x00\x00\x00\x00\x00",
         7: b"\t\x01\x00\x00",
         8: b"\t\x01\x00\x00",
         10: b"\xcd\xcc\xcc\xcc\xcc,_@",
@@ -351,13 +351,20 @@ def test_write_empty_manifest() -> None:
                 schema=test_schema,
                 output_file=io.new_output(tmp_avro_file),
                 snapshot_id=8744736658442914487,
+                avro_compression="deflate",
             ) as _:
                 pass
 
 
 @pytest.mark.parametrize("format_version", [1, 2])
+@pytest.mark.parametrize("compression", ["null", "deflate", "zstd"])
 def test_write_manifest(
-    generated_manifest_file_file_v1: str, generated_manifest_file_file_v2: str, format_version: TableVersion
+    generated_manifest_file_file_v1: str,
+    generated_manifest_file_file_v2: str,
+    format_version: TableVersion,
+    test_schema: Schema,
+    test_partition_spec: PartitionSpec,
+    compression: AvroCompressionCodec,
 ) -> None:
     io = load_file_io()
     snapshot = Snapshot(
@@ -370,23 +377,16 @@ def test_write_manifest(
     )
     demo_manifest_file = snapshot.manifests(io)[0]
     manifest_entries = demo_manifest_file.fetch_manifest_entry(io)
-    test_schema = Schema(
-        NestedField(1, "VendorID", IntegerType(), False), NestedField(2, "tpep_pickup_datetime", IntegerType(), False)
-    )
-    test_spec = PartitionSpec(
-        PartitionField(source_id=1, field_id=1, transform=IdentityTransform(), name="VendorID"),
-        PartitionField(source_id=2, field_id=2, transform=IdentityTransform(), name="tpep_pickup_datetime"),
-        spec_id=demo_manifest_file.partition_spec_id,
-    )
     with TemporaryDirectory() as tmpdir:
         tmp_avro_file = tmpdir + "/test_write_manifest.avro"
         output = io.new_output(tmp_avro_file)
         with write_manifest(
             format_version=format_version,
-            spec=test_spec,
+            spec=test_partition_spec,
             schema=test_schema,
             output_file=output,
             snapshot_id=8744736658442914487,
+            avro_compression=compression,
         ) as writer:
             for entry in manifest_entries:
                 writer.add_entry(entry)
@@ -397,7 +397,7 @@ def test_write_manifest(
 
         expected_metadata = {
             "schema": test_schema.model_dump_json(),
-            "partition-spec": """[{"source-id":1,"field-id":1,"transform":"identity","name":"VendorID"},{"source-id":2,"field-id":2,"transform":"identity","name":"tpep_pickup_datetime"}]""",
+            "partition-spec": """[{"source-id":1,"field-id":1000,"transform":"identity","name":"VendorID"},{"source-id":2,"field-id":1001,"transform":"day","name":"tpep_pickup_day"}]""",
             "partition-spec-id": str(demo_manifest_file.partition_spec_id),
             "format-version": str(format_version),
         }
@@ -416,13 +416,13 @@ def test_write_manifest(
 
         data_file = manifest_entry.data_file
 
-        assert data_file.content is DataFileContent.DATA
+        assert data_file.content == DataFileContent.DATA
         assert (
             data_file.file_path
             == "/home/iceberg/warehouse/nyc/taxis_partitioned/data/VendorID=null/00000-633-d8a4223e-dc97-45a1-86e1-adaba6e8abd7-00001.parquet"
         )
         assert data_file.file_format == FileFormat.PARQUET
-        assert data_file.partition == Record(VendorID=1, tpep_pickup_datetime=1925)
+        assert data_file.partition == Record(1, 1925)
         assert data_file.record_count == 19513
         assert data_file.file_size_in_bytes == 388872
         assert data_file.column_sizes == {
@@ -490,8 +490,8 @@ def test_write_manifest(
         }
         assert data_file.nan_value_counts == {16: 0, 17: 0, 18: 0, 19: 0, 10: 0, 11: 0, 12: 0, 13: 0, 14: 0, 15: 0}
         assert data_file.lower_bounds == {
-            2: b"2020-04-01 00:00",
-            3: b"2020-04-01 00:12",
+            2: b"\x01\x00\x00\x00\x00\x00\x00\x00",
+            3: b"\x01\x00\x00\x00\x00\x00\x00\x00",
             7: b"\x03\x00\x00\x00",
             8: b"\x01\x00\x00\x00",
             10: b"\xf6(\\\x8f\xc2\x05S\xc0",
@@ -505,8 +505,8 @@ def test_write_manifest(
             19: b"\x00\x00\x00\x00\x00\x00\x04\xc0",
         }
         assert data_file.upper_bounds == {
-            2: b"2020-04-30 23:5:",
-            3: b"2020-05-01 00:41",
+            2: b"\x06\x00\x00\x00\x00\x00\x00\x00",
+            3: b"\x06\x00\x00\x00\x00\x00\x00\x00",
             7: b"\t\x01\x00\x00",
             8: b"\t\x01\x00\x00",
             10: b"\xcd\xcc\xcc\xcc\xcc,_@",
@@ -527,11 +527,13 @@ def test_write_manifest(
 
 @pytest.mark.parametrize("format_version", [1, 2])
 @pytest.mark.parametrize("parent_snapshot_id", [19, None])
+@pytest.mark.parametrize("compression", ["null", "deflate"])
 def test_write_manifest_list(
     generated_manifest_file_file_v1: str,
     generated_manifest_file_file_v2: str,
     format_version: TableVersion,
     parent_snapshot_id: Optional[int],
+    compression: AvroCompressionCodec,
 ) -> None:
     io = load_file_io()
 
@@ -554,6 +556,7 @@ def test_write_manifest_list(
             snapshot_id=25,
             parent_snapshot_id=parent_snapshot_id,
             sequence_number=0,
+            avro_compression=compression,
         ) as writer:
             writer.add_manifests(demo_manifest_list)
         new_manifest_list = list(read_manifest_list(io.new_input(path)))
@@ -621,9 +624,9 @@ def test_write_manifest_list(
 def test_file_format_case_insensitive(raw_file_format: str, expected_file_format: FileFormat) -> None:
     if expected_file_format:
         parsed_file_format = FileFormat(raw_file_format)
-        assert (
-            parsed_file_format == expected_file_format
-        ), f"File format {raw_file_format}: {parsed_file_format} != {expected_file_format}"
+        assert parsed_file_format == expected_file_format, (
+            f"File format {raw_file_format}: {parsed_file_format} != {expected_file_format}"
+        )
     else:
         with pytest.raises(ValueError):
             _ = FileFormat(raw_file_format)
